# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities
from . import outputs
from ._inputs import *

__all__ = ['WafPolicyArgs', 'WafPolicy']

@pulumi.input_type
class WafPolicyArgs:
    def __init__(__self__, *,
                 name: pulumi.Input[_builtins.str],
                 template_name: pulumi.Input[_builtins.str],
                 application_language: Optional[pulumi.Input[_builtins.str]] = None,
                 case_insensitive: Optional[pulumi.Input[_builtins.bool]] = None,
                 description: Optional[pulumi.Input[_builtins.str]] = None,
                 enable_passivemode: Optional[pulumi.Input[_builtins.bool]] = None,
                 enforcement_mode: Optional[pulumi.Input[_builtins.str]] = None,
                 file_types: Optional[pulumi.Input[Sequence[pulumi.Input['WafPolicyFileTypeArgs']]]] = None,
                 graphql_profiles: Optional[pulumi.Input[Sequence[pulumi.Input['WafPolicyGraphqlProfileArgs']]]] = None,
                 host_names: Optional[pulumi.Input[Sequence[pulumi.Input['WafPolicyHostNameArgs']]]] = None,
                 ip_exceptions: Optional[pulumi.Input[Sequence[pulumi.Input['WafPolicyIpExceptionArgs']]]] = None,
                 modifications: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 open_api_files: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 parameters: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 partition: Optional[pulumi.Input[_builtins.str]] = None,
                 policy_builders: Optional[pulumi.Input[Sequence[pulumi.Input['WafPolicyPolicyBuilderArgs']]]] = None,
                 policy_id: Optional[pulumi.Input[_builtins.str]] = None,
                 policy_import_json: Optional[pulumi.Input[_builtins.str]] = None,
                 protocol_independent: Optional[pulumi.Input[_builtins.bool]] = None,
                 server_technologies: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 signature_sets: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 signatures: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 signatures_settings: Optional[pulumi.Input[Sequence[pulumi.Input['WafPolicySignaturesSettingArgs']]]] = None,
                 template_link: Optional[pulumi.Input[_builtins.str]] = None,
                 type: Optional[pulumi.Input[_builtins.str]] = None,
                 urls: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None):
        """
        The set of arguments for constructing a WafPolicy resource.
        :param pulumi.Input[_builtins.str] name: The unique user-given name of the policy. Policy names cannot contain spaces or special characters. Allowed characters are a-z, A-Z, 0-9, dot, dash (-), colon (:) and underscore (_).
        :param pulumi.Input[_builtins.str] template_name: Specifies the name of the template used for the policy creation.
        :param pulumi.Input[_builtins.str] application_language: The character encoding for the web application. The character encoding determines how the policy processes the character sets. The default is `utf-8`
        :param pulumi.Input[_builtins.bool] case_insensitive: Specifies whether the security policy treats microservice URLs, file types, URLs, and parameters as case sensitive or not. When this setting is enabled, the system stores these security policy elements in lowercase in the security policy configuration
        :param pulumi.Input[_builtins.str] description: Specifies the description of the policy.
        :param pulumi.Input[_builtins.bool] enable_passivemode: Passive Mode allows the policy to be associated with a Performance L4 Virtual Server (using a FastL4 profile). With FastL4, traffic is analyzed but is not modified in any way.
        :param pulumi.Input[_builtins.str] enforcement_mode: How the system processes a request that triggers a security policy violation
        :param pulumi.Input[Sequence[pulumi.Input['WafPolicyFileTypeArgs']]] file_types: `file_types` takes list of file-types options to be used for policy builder.
               See file types below for more details.
        :param pulumi.Input[Sequence[pulumi.Input['WafPolicyGraphqlProfileArgs']]] graphql_profiles: `graphql_profiles` takes list of graphql profile options to be used for policy builder.
               See graphql profiles below for more details.
        :param pulumi.Input[Sequence[pulumi.Input['WafPolicyHostNameArgs']]] host_names: specify the list of host name that is used to access the application
        :param pulumi.Input[Sequence[pulumi.Input['WafPolicyIpExceptionArgs']]] ip_exceptions: `ip_exceptions` takes list of IP address exception,An IP address exception is an IP address that you want the system to treat in a specific way for a security policy.For example, you can specify IP addresses from which the system should always trust traffic.
               See IP Exceptions below for more details.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] modifications: the modifications section includes actions that modify the declarative policy as it is defined in the adjustments
               section. The modifications section is updated manually, with the changes generally driven by the learning suggestions
               provided by the BIG-IP.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] open_api_files: This section defines the Link for open api files on the policy.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] parameters: This section defines parameters that the security policy permits in requests.
        :param pulumi.Input[_builtins.str] partition: Specifies the partition of the policy. Default is `Common`
        :param pulumi.Input[Sequence[pulumi.Input['WafPolicyPolicyBuilderArgs']]] policy_builders: `policy_builder` block will provide `learning_mode` options to be used for policy builder.
               See policy builder below for more details.
        :param pulumi.Input[_builtins.str] policy_id: The id of the A.WAF Policy as it would be calculated on the BIG-IP.
        :param pulumi.Input[_builtins.str] policy_import_json: The payload of the WAF Policy to be used for IMPORT on to BIG-IP.
        :param pulumi.Input[_builtins.bool] protocol_independent: When creating a security policy, you can determine whether a security policy differentiates between HTTP and HTTPS URLs. If enabled, the security policy differentiates between HTTP and HTTPS URLs. If disabled, the security policy configures URLs without specifying a specific protocol. This is useful for applications that behave the same for HTTP and HTTPS, and it keeps the security policy from including the same URL twice.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] server_technologies: The server technology is a server-side application, framework, web server or operating system type that is configured in the policy in order to adapt the policy to the checks needed for the respective technology.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] signature_sets: Defines behavior when signatures found within a signature-set are detected in a request. Settings are culmulative, so if a signature is found in any set with block enabled, that signature will have block enabled.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] signatures: This section defines the properties of a signature on the policy.
        :param pulumi.Input[Sequence[pulumi.Input['WafPolicySignaturesSettingArgs']]] signatures_settings: bulk signature setting
        :param pulumi.Input[_builtins.str] template_link: Specifies the Link of the template used for the policy creation.
        :param pulumi.Input[_builtins.str] type: The type of policy you want to create. The default policy type is `security`.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] urls: In a security policy, you can manually specify the HTTP URLs that are allowed (or disallowed) in traffic to the web application being protected. If you are using automatic policy building (and the policy includes learning URLs), the system can determine which URLs to add, based on legitimate traffic.
        """
        pulumi.set(__self__, "name", name)
        pulumi.set(__self__, "template_name", template_name)
        if application_language is not None:
            pulumi.set(__self__, "application_language", application_language)
        if case_insensitive is not None:
            pulumi.set(__self__, "case_insensitive", case_insensitive)
        if description is not None:
            pulumi.set(__self__, "description", description)
        if enable_passivemode is not None:
            pulumi.set(__self__, "enable_passivemode", enable_passivemode)
        if enforcement_mode is not None:
            pulumi.set(__self__, "enforcement_mode", enforcement_mode)
        if file_types is not None:
            pulumi.set(__self__, "file_types", file_types)
        if graphql_profiles is not None:
            pulumi.set(__self__, "graphql_profiles", graphql_profiles)
        if host_names is not None:
            pulumi.set(__self__, "host_names", host_names)
        if ip_exceptions is not None:
            pulumi.set(__self__, "ip_exceptions", ip_exceptions)
        if modifications is not None:
            pulumi.set(__self__, "modifications", modifications)
        if open_api_files is not None:
            pulumi.set(__self__, "open_api_files", open_api_files)
        if parameters is not None:
            pulumi.set(__self__, "parameters", parameters)
        if partition is not None:
            pulumi.set(__self__, "partition", partition)
        if policy_builders is not None:
            pulumi.set(__self__, "policy_builders", policy_builders)
        if policy_id is not None:
            pulumi.set(__self__, "policy_id", policy_id)
        if policy_import_json is not None:
            pulumi.set(__self__, "policy_import_json", policy_import_json)
        if protocol_independent is not None:
            pulumi.set(__self__, "protocol_independent", protocol_independent)
        if server_technologies is not None:
            pulumi.set(__self__, "server_technologies", server_technologies)
        if signature_sets is not None:
            pulumi.set(__self__, "signature_sets", signature_sets)
        if signatures is not None:
            pulumi.set(__self__, "signatures", signatures)
        if signatures_settings is not None:
            pulumi.set(__self__, "signatures_settings", signatures_settings)
        if template_link is not None:
            pulumi.set(__self__, "template_link", template_link)
        if type is not None:
            pulumi.set(__self__, "type", type)
        if urls is not None:
            pulumi.set(__self__, "urls", urls)

    @_builtins.property
    @pulumi.getter
    def name(self) -> pulumi.Input[_builtins.str]:
        """
        The unique user-given name of the policy. Policy names cannot contain spaces or special characters. Allowed characters are a-z, A-Z, 0-9, dot, dash (-), colon (:) and underscore (_).
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "name", value)

    @_builtins.property
    @pulumi.getter(name="templateName")
    def template_name(self) -> pulumi.Input[_builtins.str]:
        """
        Specifies the name of the template used for the policy creation.
        """
        return pulumi.get(self, "template_name")

    @template_name.setter
    def template_name(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "template_name", value)

    @_builtins.property
    @pulumi.getter(name="applicationLanguage")
    def application_language(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The character encoding for the web application. The character encoding determines how the policy processes the character sets. The default is `utf-8`
        """
        return pulumi.get(self, "application_language")

    @application_language.setter
    def application_language(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "application_language", value)

    @_builtins.property
    @pulumi.getter(name="caseInsensitive")
    def case_insensitive(self) -> Optional[pulumi.Input[_builtins.bool]]:
        """
        Specifies whether the security policy treats microservice URLs, file types, URLs, and parameters as case sensitive or not. When this setting is enabled, the system stores these security policy elements in lowercase in the security policy configuration
        """
        return pulumi.get(self, "case_insensitive")

    @case_insensitive.setter
    def case_insensitive(self, value: Optional[pulumi.Input[_builtins.bool]]):
        pulumi.set(self, "case_insensitive", value)

    @_builtins.property
    @pulumi.getter
    def description(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Specifies the description of the policy.
        """
        return pulumi.get(self, "description")

    @description.setter
    def description(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "description", value)

    @_builtins.property
    @pulumi.getter(name="enablePassivemode")
    def enable_passivemode(self) -> Optional[pulumi.Input[_builtins.bool]]:
        """
        Passive Mode allows the policy to be associated with a Performance L4 Virtual Server (using a FastL4 profile). With FastL4, traffic is analyzed but is not modified in any way.
        """
        return pulumi.get(self, "enable_passivemode")

    @enable_passivemode.setter
    def enable_passivemode(self, value: Optional[pulumi.Input[_builtins.bool]]):
        pulumi.set(self, "enable_passivemode", value)

    @_builtins.property
    @pulumi.getter(name="enforcementMode")
    def enforcement_mode(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        How the system processes a request that triggers a security policy violation
        """
        return pulumi.get(self, "enforcement_mode")

    @enforcement_mode.setter
    def enforcement_mode(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "enforcement_mode", value)

    @_builtins.property
    @pulumi.getter(name="fileTypes")
    def file_types(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['WafPolicyFileTypeArgs']]]]:
        """
        `file_types` takes list of file-types options to be used for policy builder.
        See file types below for more details.
        """
        return pulumi.get(self, "file_types")

    @file_types.setter
    def file_types(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['WafPolicyFileTypeArgs']]]]):
        pulumi.set(self, "file_types", value)

    @_builtins.property
    @pulumi.getter(name="graphqlProfiles")
    def graphql_profiles(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['WafPolicyGraphqlProfileArgs']]]]:
        """
        `graphql_profiles` takes list of graphql profile options to be used for policy builder.
        See graphql profiles below for more details.
        """
        return pulumi.get(self, "graphql_profiles")

    @graphql_profiles.setter
    def graphql_profiles(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['WafPolicyGraphqlProfileArgs']]]]):
        pulumi.set(self, "graphql_profiles", value)

    @_builtins.property
    @pulumi.getter(name="hostNames")
    def host_names(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['WafPolicyHostNameArgs']]]]:
        """
        specify the list of host name that is used to access the application
        """
        return pulumi.get(self, "host_names")

    @host_names.setter
    def host_names(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['WafPolicyHostNameArgs']]]]):
        pulumi.set(self, "host_names", value)

    @_builtins.property
    @pulumi.getter(name="ipExceptions")
    def ip_exceptions(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['WafPolicyIpExceptionArgs']]]]:
        """
        `ip_exceptions` takes list of IP address exception,An IP address exception is an IP address that you want the system to treat in a specific way for a security policy.For example, you can specify IP addresses from which the system should always trust traffic.
        See IP Exceptions below for more details.
        """
        return pulumi.get(self, "ip_exceptions")

    @ip_exceptions.setter
    def ip_exceptions(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['WafPolicyIpExceptionArgs']]]]):
        pulumi.set(self, "ip_exceptions", value)

    @_builtins.property
    @pulumi.getter
    def modifications(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        the modifications section includes actions that modify the declarative policy as it is defined in the adjustments
        section. The modifications section is updated manually, with the changes generally driven by the learning suggestions
        provided by the BIG-IP.
        """
        return pulumi.get(self, "modifications")

    @modifications.setter
    def modifications(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "modifications", value)

    @_builtins.property
    @pulumi.getter(name="openApiFiles")
    def open_api_files(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        This section defines the Link for open api files on the policy.
        """
        return pulumi.get(self, "open_api_files")

    @open_api_files.setter
    def open_api_files(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "open_api_files", value)

    @_builtins.property
    @pulumi.getter
    def parameters(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        This section defines parameters that the security policy permits in requests.
        """
        return pulumi.get(self, "parameters")

    @parameters.setter
    def parameters(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "parameters", value)

    @_builtins.property
    @pulumi.getter
    def partition(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Specifies the partition of the policy. Default is `Common`
        """
        return pulumi.get(self, "partition")

    @partition.setter
    def partition(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "partition", value)

    @_builtins.property
    @pulumi.getter(name="policyBuilders")
    def policy_builders(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['WafPolicyPolicyBuilderArgs']]]]:
        """
        `policy_builder` block will provide `learning_mode` options to be used for policy builder.
        See policy builder below for more details.
        """
        return pulumi.get(self, "policy_builders")

    @policy_builders.setter
    def policy_builders(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['WafPolicyPolicyBuilderArgs']]]]):
        pulumi.set(self, "policy_builders", value)

    @_builtins.property
    @pulumi.getter(name="policyId")
    def policy_id(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The id of the A.WAF Policy as it would be calculated on the BIG-IP.
        """
        return pulumi.get(self, "policy_id")

    @policy_id.setter
    def policy_id(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "policy_id", value)

    @_builtins.property
    @pulumi.getter(name="policyImportJson")
    def policy_import_json(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The payload of the WAF Policy to be used for IMPORT on to BIG-IP.
        """
        return pulumi.get(self, "policy_import_json")

    @policy_import_json.setter
    def policy_import_json(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "policy_import_json", value)

    @_builtins.property
    @pulumi.getter(name="protocolIndependent")
    def protocol_independent(self) -> Optional[pulumi.Input[_builtins.bool]]:
        """
        When creating a security policy, you can determine whether a security policy differentiates between HTTP and HTTPS URLs. If enabled, the security policy differentiates between HTTP and HTTPS URLs. If disabled, the security policy configures URLs without specifying a specific protocol. This is useful for applications that behave the same for HTTP and HTTPS, and it keeps the security policy from including the same URL twice.
        """
        return pulumi.get(self, "protocol_independent")

    @protocol_independent.setter
    def protocol_independent(self, value: Optional[pulumi.Input[_builtins.bool]]):
        pulumi.set(self, "protocol_independent", value)

    @_builtins.property
    @pulumi.getter(name="serverTechnologies")
    def server_technologies(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        The server technology is a server-side application, framework, web server or operating system type that is configured in the policy in order to adapt the policy to the checks needed for the respective technology.
        """
        return pulumi.get(self, "server_technologies")

    @server_technologies.setter
    def server_technologies(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "server_technologies", value)

    @_builtins.property
    @pulumi.getter(name="signatureSets")
    def signature_sets(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        Defines behavior when signatures found within a signature-set are detected in a request. Settings are culmulative, so if a signature is found in any set with block enabled, that signature will have block enabled.
        """
        return pulumi.get(self, "signature_sets")

    @signature_sets.setter
    def signature_sets(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "signature_sets", value)

    @_builtins.property
    @pulumi.getter
    def signatures(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        This section defines the properties of a signature on the policy.
        """
        return pulumi.get(self, "signatures")

    @signatures.setter
    def signatures(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "signatures", value)

    @_builtins.property
    @pulumi.getter(name="signaturesSettings")
    def signatures_settings(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['WafPolicySignaturesSettingArgs']]]]:
        """
        bulk signature setting
        """
        return pulumi.get(self, "signatures_settings")

    @signatures_settings.setter
    def signatures_settings(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['WafPolicySignaturesSettingArgs']]]]):
        pulumi.set(self, "signatures_settings", value)

    @_builtins.property
    @pulumi.getter(name="templateLink")
    def template_link(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Specifies the Link of the template used for the policy creation.
        """
        return pulumi.get(self, "template_link")

    @template_link.setter
    def template_link(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "template_link", value)

    @_builtins.property
    @pulumi.getter
    def type(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The type of policy you want to create. The default policy type is `security`.
        """
        return pulumi.get(self, "type")

    @type.setter
    def type(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "type", value)

    @_builtins.property
    @pulumi.getter
    def urls(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        In a security policy, you can manually specify the HTTP URLs that are allowed (or disallowed) in traffic to the web application being protected. If you are using automatic policy building (and the policy includes learning URLs), the system can determine which URLs to add, based on legitimate traffic.
        """
        return pulumi.get(self, "urls")

    @urls.setter
    def urls(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "urls", value)


@pulumi.input_type
class _WafPolicyState:
    def __init__(__self__, *,
                 application_language: Optional[pulumi.Input[_builtins.str]] = None,
                 case_insensitive: Optional[pulumi.Input[_builtins.bool]] = None,
                 description: Optional[pulumi.Input[_builtins.str]] = None,
                 enable_passivemode: Optional[pulumi.Input[_builtins.bool]] = None,
                 enforcement_mode: Optional[pulumi.Input[_builtins.str]] = None,
                 file_types: Optional[pulumi.Input[Sequence[pulumi.Input['WafPolicyFileTypeArgs']]]] = None,
                 graphql_profiles: Optional[pulumi.Input[Sequence[pulumi.Input['WafPolicyGraphqlProfileArgs']]]] = None,
                 host_names: Optional[pulumi.Input[Sequence[pulumi.Input['WafPolicyHostNameArgs']]]] = None,
                 ip_exceptions: Optional[pulumi.Input[Sequence[pulumi.Input['WafPolicyIpExceptionArgs']]]] = None,
                 modifications: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 name: Optional[pulumi.Input[_builtins.str]] = None,
                 open_api_files: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 parameters: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 partition: Optional[pulumi.Input[_builtins.str]] = None,
                 policy_builders: Optional[pulumi.Input[Sequence[pulumi.Input['WafPolicyPolicyBuilderArgs']]]] = None,
                 policy_export_json: Optional[pulumi.Input[_builtins.str]] = None,
                 policy_id: Optional[pulumi.Input[_builtins.str]] = None,
                 policy_import_json: Optional[pulumi.Input[_builtins.str]] = None,
                 protocol_independent: Optional[pulumi.Input[_builtins.bool]] = None,
                 server_technologies: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 signature_sets: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 signatures: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 signatures_settings: Optional[pulumi.Input[Sequence[pulumi.Input['WafPolicySignaturesSettingArgs']]]] = None,
                 template_link: Optional[pulumi.Input[_builtins.str]] = None,
                 template_name: Optional[pulumi.Input[_builtins.str]] = None,
                 type: Optional[pulumi.Input[_builtins.str]] = None,
                 urls: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None):
        """
        Input properties used for looking up and filtering WafPolicy resources.
        :param pulumi.Input[_builtins.str] application_language: The character encoding for the web application. The character encoding determines how the policy processes the character sets. The default is `utf-8`
        :param pulumi.Input[_builtins.bool] case_insensitive: Specifies whether the security policy treats microservice URLs, file types, URLs, and parameters as case sensitive or not. When this setting is enabled, the system stores these security policy elements in lowercase in the security policy configuration
        :param pulumi.Input[_builtins.str] description: Specifies the description of the policy.
        :param pulumi.Input[_builtins.bool] enable_passivemode: Passive Mode allows the policy to be associated with a Performance L4 Virtual Server (using a FastL4 profile). With FastL4, traffic is analyzed but is not modified in any way.
        :param pulumi.Input[_builtins.str] enforcement_mode: How the system processes a request that triggers a security policy violation
        :param pulumi.Input[Sequence[pulumi.Input['WafPolicyFileTypeArgs']]] file_types: `file_types` takes list of file-types options to be used for policy builder.
               See file types below for more details.
        :param pulumi.Input[Sequence[pulumi.Input['WafPolicyGraphqlProfileArgs']]] graphql_profiles: `graphql_profiles` takes list of graphql profile options to be used for policy builder.
               See graphql profiles below for more details.
        :param pulumi.Input[Sequence[pulumi.Input['WafPolicyHostNameArgs']]] host_names: specify the list of host name that is used to access the application
        :param pulumi.Input[Sequence[pulumi.Input['WafPolicyIpExceptionArgs']]] ip_exceptions: `ip_exceptions` takes list of IP address exception,An IP address exception is an IP address that you want the system to treat in a specific way for a security policy.For example, you can specify IP addresses from which the system should always trust traffic.
               See IP Exceptions below for more details.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] modifications: the modifications section includes actions that modify the declarative policy as it is defined in the adjustments
               section. The modifications section is updated manually, with the changes generally driven by the learning suggestions
               provided by the BIG-IP.
        :param pulumi.Input[_builtins.str] name: The unique user-given name of the policy. Policy names cannot contain spaces or special characters. Allowed characters are a-z, A-Z, 0-9, dot, dash (-), colon (:) and underscore (_).
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] open_api_files: This section defines the Link for open api files on the policy.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] parameters: This section defines parameters that the security policy permits in requests.
        :param pulumi.Input[_builtins.str] partition: Specifies the partition of the policy. Default is `Common`
        :param pulumi.Input[Sequence[pulumi.Input['WafPolicyPolicyBuilderArgs']]] policy_builders: `policy_builder` block will provide `learning_mode` options to be used for policy builder.
               See policy builder below for more details.
        :param pulumi.Input[_builtins.str] policy_export_json: Exported WAF policy deployed on BIGIP.
        :param pulumi.Input[_builtins.str] policy_id: The id of the A.WAF Policy as it would be calculated on the BIG-IP.
        :param pulumi.Input[_builtins.str] policy_import_json: The payload of the WAF Policy to be used for IMPORT on to BIG-IP.
        :param pulumi.Input[_builtins.bool] protocol_independent: When creating a security policy, you can determine whether a security policy differentiates between HTTP and HTTPS URLs. If enabled, the security policy differentiates between HTTP and HTTPS URLs. If disabled, the security policy configures URLs without specifying a specific protocol. This is useful for applications that behave the same for HTTP and HTTPS, and it keeps the security policy from including the same URL twice.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] server_technologies: The server technology is a server-side application, framework, web server or operating system type that is configured in the policy in order to adapt the policy to the checks needed for the respective technology.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] signature_sets: Defines behavior when signatures found within a signature-set are detected in a request. Settings are culmulative, so if a signature is found in any set with block enabled, that signature will have block enabled.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] signatures: This section defines the properties of a signature on the policy.
        :param pulumi.Input[Sequence[pulumi.Input['WafPolicySignaturesSettingArgs']]] signatures_settings: bulk signature setting
        :param pulumi.Input[_builtins.str] template_link: Specifies the Link of the template used for the policy creation.
        :param pulumi.Input[_builtins.str] template_name: Specifies the name of the template used for the policy creation.
        :param pulumi.Input[_builtins.str] type: The type of policy you want to create. The default policy type is `security`.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] urls: In a security policy, you can manually specify the HTTP URLs that are allowed (or disallowed) in traffic to the web application being protected. If you are using automatic policy building (and the policy includes learning URLs), the system can determine which URLs to add, based on legitimate traffic.
        """
        if application_language is not None:
            pulumi.set(__self__, "application_language", application_language)
        if case_insensitive is not None:
            pulumi.set(__self__, "case_insensitive", case_insensitive)
        if description is not None:
            pulumi.set(__self__, "description", description)
        if enable_passivemode is not None:
            pulumi.set(__self__, "enable_passivemode", enable_passivemode)
        if enforcement_mode is not None:
            pulumi.set(__self__, "enforcement_mode", enforcement_mode)
        if file_types is not None:
            pulumi.set(__self__, "file_types", file_types)
        if graphql_profiles is not None:
            pulumi.set(__self__, "graphql_profiles", graphql_profiles)
        if host_names is not None:
            pulumi.set(__self__, "host_names", host_names)
        if ip_exceptions is not None:
            pulumi.set(__self__, "ip_exceptions", ip_exceptions)
        if modifications is not None:
            pulumi.set(__self__, "modifications", modifications)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if open_api_files is not None:
            pulumi.set(__self__, "open_api_files", open_api_files)
        if parameters is not None:
            pulumi.set(__self__, "parameters", parameters)
        if partition is not None:
            pulumi.set(__self__, "partition", partition)
        if policy_builders is not None:
            pulumi.set(__self__, "policy_builders", policy_builders)
        if policy_export_json is not None:
            pulumi.set(__self__, "policy_export_json", policy_export_json)
        if policy_id is not None:
            pulumi.set(__self__, "policy_id", policy_id)
        if policy_import_json is not None:
            pulumi.set(__self__, "policy_import_json", policy_import_json)
        if protocol_independent is not None:
            pulumi.set(__self__, "protocol_independent", protocol_independent)
        if server_technologies is not None:
            pulumi.set(__self__, "server_technologies", server_technologies)
        if signature_sets is not None:
            pulumi.set(__self__, "signature_sets", signature_sets)
        if signatures is not None:
            pulumi.set(__self__, "signatures", signatures)
        if signatures_settings is not None:
            pulumi.set(__self__, "signatures_settings", signatures_settings)
        if template_link is not None:
            pulumi.set(__self__, "template_link", template_link)
        if template_name is not None:
            pulumi.set(__self__, "template_name", template_name)
        if type is not None:
            pulumi.set(__self__, "type", type)
        if urls is not None:
            pulumi.set(__self__, "urls", urls)

    @_builtins.property
    @pulumi.getter(name="applicationLanguage")
    def application_language(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The character encoding for the web application. The character encoding determines how the policy processes the character sets. The default is `utf-8`
        """
        return pulumi.get(self, "application_language")

    @application_language.setter
    def application_language(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "application_language", value)

    @_builtins.property
    @pulumi.getter(name="caseInsensitive")
    def case_insensitive(self) -> Optional[pulumi.Input[_builtins.bool]]:
        """
        Specifies whether the security policy treats microservice URLs, file types, URLs, and parameters as case sensitive or not. When this setting is enabled, the system stores these security policy elements in lowercase in the security policy configuration
        """
        return pulumi.get(self, "case_insensitive")

    @case_insensitive.setter
    def case_insensitive(self, value: Optional[pulumi.Input[_builtins.bool]]):
        pulumi.set(self, "case_insensitive", value)

    @_builtins.property
    @pulumi.getter
    def description(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Specifies the description of the policy.
        """
        return pulumi.get(self, "description")

    @description.setter
    def description(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "description", value)

    @_builtins.property
    @pulumi.getter(name="enablePassivemode")
    def enable_passivemode(self) -> Optional[pulumi.Input[_builtins.bool]]:
        """
        Passive Mode allows the policy to be associated with a Performance L4 Virtual Server (using a FastL4 profile). With FastL4, traffic is analyzed but is not modified in any way.
        """
        return pulumi.get(self, "enable_passivemode")

    @enable_passivemode.setter
    def enable_passivemode(self, value: Optional[pulumi.Input[_builtins.bool]]):
        pulumi.set(self, "enable_passivemode", value)

    @_builtins.property
    @pulumi.getter(name="enforcementMode")
    def enforcement_mode(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        How the system processes a request that triggers a security policy violation
        """
        return pulumi.get(self, "enforcement_mode")

    @enforcement_mode.setter
    def enforcement_mode(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "enforcement_mode", value)

    @_builtins.property
    @pulumi.getter(name="fileTypes")
    def file_types(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['WafPolicyFileTypeArgs']]]]:
        """
        `file_types` takes list of file-types options to be used for policy builder.
        See file types below for more details.
        """
        return pulumi.get(self, "file_types")

    @file_types.setter
    def file_types(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['WafPolicyFileTypeArgs']]]]):
        pulumi.set(self, "file_types", value)

    @_builtins.property
    @pulumi.getter(name="graphqlProfiles")
    def graphql_profiles(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['WafPolicyGraphqlProfileArgs']]]]:
        """
        `graphql_profiles` takes list of graphql profile options to be used for policy builder.
        See graphql profiles below for more details.
        """
        return pulumi.get(self, "graphql_profiles")

    @graphql_profiles.setter
    def graphql_profiles(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['WafPolicyGraphqlProfileArgs']]]]):
        pulumi.set(self, "graphql_profiles", value)

    @_builtins.property
    @pulumi.getter(name="hostNames")
    def host_names(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['WafPolicyHostNameArgs']]]]:
        """
        specify the list of host name that is used to access the application
        """
        return pulumi.get(self, "host_names")

    @host_names.setter
    def host_names(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['WafPolicyHostNameArgs']]]]):
        pulumi.set(self, "host_names", value)

    @_builtins.property
    @pulumi.getter(name="ipExceptions")
    def ip_exceptions(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['WafPolicyIpExceptionArgs']]]]:
        """
        `ip_exceptions` takes list of IP address exception,An IP address exception is an IP address that you want the system to treat in a specific way for a security policy.For example, you can specify IP addresses from which the system should always trust traffic.
        See IP Exceptions below for more details.
        """
        return pulumi.get(self, "ip_exceptions")

    @ip_exceptions.setter
    def ip_exceptions(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['WafPolicyIpExceptionArgs']]]]):
        pulumi.set(self, "ip_exceptions", value)

    @_builtins.property
    @pulumi.getter
    def modifications(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        the modifications section includes actions that modify the declarative policy as it is defined in the adjustments
        section. The modifications section is updated manually, with the changes generally driven by the learning suggestions
        provided by the BIG-IP.
        """
        return pulumi.get(self, "modifications")

    @modifications.setter
    def modifications(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "modifications", value)

    @_builtins.property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The unique user-given name of the policy. Policy names cannot contain spaces or special characters. Allowed characters are a-z, A-Z, 0-9, dot, dash (-), colon (:) and underscore (_).
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "name", value)

    @_builtins.property
    @pulumi.getter(name="openApiFiles")
    def open_api_files(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        This section defines the Link for open api files on the policy.
        """
        return pulumi.get(self, "open_api_files")

    @open_api_files.setter
    def open_api_files(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "open_api_files", value)

    @_builtins.property
    @pulumi.getter
    def parameters(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        This section defines parameters that the security policy permits in requests.
        """
        return pulumi.get(self, "parameters")

    @parameters.setter
    def parameters(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "parameters", value)

    @_builtins.property
    @pulumi.getter
    def partition(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Specifies the partition of the policy. Default is `Common`
        """
        return pulumi.get(self, "partition")

    @partition.setter
    def partition(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "partition", value)

    @_builtins.property
    @pulumi.getter(name="policyBuilders")
    def policy_builders(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['WafPolicyPolicyBuilderArgs']]]]:
        """
        `policy_builder` block will provide `learning_mode` options to be used for policy builder.
        See policy builder below for more details.
        """
        return pulumi.get(self, "policy_builders")

    @policy_builders.setter
    def policy_builders(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['WafPolicyPolicyBuilderArgs']]]]):
        pulumi.set(self, "policy_builders", value)

    @_builtins.property
    @pulumi.getter(name="policyExportJson")
    def policy_export_json(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Exported WAF policy deployed on BIGIP.
        """
        return pulumi.get(self, "policy_export_json")

    @policy_export_json.setter
    def policy_export_json(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "policy_export_json", value)

    @_builtins.property
    @pulumi.getter(name="policyId")
    def policy_id(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The id of the A.WAF Policy as it would be calculated on the BIG-IP.
        """
        return pulumi.get(self, "policy_id")

    @policy_id.setter
    def policy_id(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "policy_id", value)

    @_builtins.property
    @pulumi.getter(name="policyImportJson")
    def policy_import_json(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The payload of the WAF Policy to be used for IMPORT on to BIG-IP.
        """
        return pulumi.get(self, "policy_import_json")

    @policy_import_json.setter
    def policy_import_json(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "policy_import_json", value)

    @_builtins.property
    @pulumi.getter(name="protocolIndependent")
    def protocol_independent(self) -> Optional[pulumi.Input[_builtins.bool]]:
        """
        When creating a security policy, you can determine whether a security policy differentiates between HTTP and HTTPS URLs. If enabled, the security policy differentiates between HTTP and HTTPS URLs. If disabled, the security policy configures URLs without specifying a specific protocol. This is useful for applications that behave the same for HTTP and HTTPS, and it keeps the security policy from including the same URL twice.
        """
        return pulumi.get(self, "protocol_independent")

    @protocol_independent.setter
    def protocol_independent(self, value: Optional[pulumi.Input[_builtins.bool]]):
        pulumi.set(self, "protocol_independent", value)

    @_builtins.property
    @pulumi.getter(name="serverTechnologies")
    def server_technologies(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        The server technology is a server-side application, framework, web server or operating system type that is configured in the policy in order to adapt the policy to the checks needed for the respective technology.
        """
        return pulumi.get(self, "server_technologies")

    @server_technologies.setter
    def server_technologies(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "server_technologies", value)

    @_builtins.property
    @pulumi.getter(name="signatureSets")
    def signature_sets(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        Defines behavior when signatures found within a signature-set are detected in a request. Settings are culmulative, so if a signature is found in any set with block enabled, that signature will have block enabled.
        """
        return pulumi.get(self, "signature_sets")

    @signature_sets.setter
    def signature_sets(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "signature_sets", value)

    @_builtins.property
    @pulumi.getter
    def signatures(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        This section defines the properties of a signature on the policy.
        """
        return pulumi.get(self, "signatures")

    @signatures.setter
    def signatures(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "signatures", value)

    @_builtins.property
    @pulumi.getter(name="signaturesSettings")
    def signatures_settings(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['WafPolicySignaturesSettingArgs']]]]:
        """
        bulk signature setting
        """
        return pulumi.get(self, "signatures_settings")

    @signatures_settings.setter
    def signatures_settings(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['WafPolicySignaturesSettingArgs']]]]):
        pulumi.set(self, "signatures_settings", value)

    @_builtins.property
    @pulumi.getter(name="templateLink")
    def template_link(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Specifies the Link of the template used for the policy creation.
        """
        return pulumi.get(self, "template_link")

    @template_link.setter
    def template_link(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "template_link", value)

    @_builtins.property
    @pulumi.getter(name="templateName")
    def template_name(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Specifies the name of the template used for the policy creation.
        """
        return pulumi.get(self, "template_name")

    @template_name.setter
    def template_name(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "template_name", value)

    @_builtins.property
    @pulumi.getter
    def type(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The type of policy you want to create. The default policy type is `security`.
        """
        return pulumi.get(self, "type")

    @type.setter
    def type(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "type", value)

    @_builtins.property
    @pulumi.getter
    def urls(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        In a security policy, you can manually specify the HTTP URLs that are allowed (or disallowed) in traffic to the web application being protected. If you are using automatic policy building (and the policy includes learning URLs), the system can determine which URLs to add, based on legitimate traffic.
        """
        return pulumi.get(self, "urls")

    @urls.setter
    def urls(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "urls", value)


@pulumi.type_token("f5bigip:index/wafPolicy:WafPolicy")
class WafPolicy(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 application_language: Optional[pulumi.Input[_builtins.str]] = None,
                 case_insensitive: Optional[pulumi.Input[_builtins.bool]] = None,
                 description: Optional[pulumi.Input[_builtins.str]] = None,
                 enable_passivemode: Optional[pulumi.Input[_builtins.bool]] = None,
                 enforcement_mode: Optional[pulumi.Input[_builtins.str]] = None,
                 file_types: Optional[pulumi.Input[Sequence[pulumi.Input[Union['WafPolicyFileTypeArgs', 'WafPolicyFileTypeArgsDict']]]]] = None,
                 graphql_profiles: Optional[pulumi.Input[Sequence[pulumi.Input[Union['WafPolicyGraphqlProfileArgs', 'WafPolicyGraphqlProfileArgsDict']]]]] = None,
                 host_names: Optional[pulumi.Input[Sequence[pulumi.Input[Union['WafPolicyHostNameArgs', 'WafPolicyHostNameArgsDict']]]]] = None,
                 ip_exceptions: Optional[pulumi.Input[Sequence[pulumi.Input[Union['WafPolicyIpExceptionArgs', 'WafPolicyIpExceptionArgsDict']]]]] = None,
                 modifications: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 name: Optional[pulumi.Input[_builtins.str]] = None,
                 open_api_files: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 parameters: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 partition: Optional[pulumi.Input[_builtins.str]] = None,
                 policy_builders: Optional[pulumi.Input[Sequence[pulumi.Input[Union['WafPolicyPolicyBuilderArgs', 'WafPolicyPolicyBuilderArgsDict']]]]] = None,
                 policy_id: Optional[pulumi.Input[_builtins.str]] = None,
                 policy_import_json: Optional[pulumi.Input[_builtins.str]] = None,
                 protocol_independent: Optional[pulumi.Input[_builtins.bool]] = None,
                 server_technologies: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 signature_sets: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 signatures: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 signatures_settings: Optional[pulumi.Input[Sequence[pulumi.Input[Union['WafPolicySignaturesSettingArgs', 'WafPolicySignaturesSettingArgsDict']]]]] = None,
                 template_link: Optional[pulumi.Input[_builtins.str]] = None,
                 template_name: Optional[pulumi.Input[_builtins.str]] = None,
                 type: Optional[pulumi.Input[_builtins.str]] = None,
                 urls: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 __props__=None):
        """
        ## Example Usage

        ```python
        import pulumi
        import pulumi_f5bigip as f5bigip

        param1 = f5bigip.ssl.get_waf_entity_parameter(name="Param1",
            type="explicit",
            data_type="alpha-numeric",
            perform_staging=True)
        param2 = f5bigip.ssl.get_waf_entity_parameter(name="Param2",
            type="explicit",
            data_type="alpha-numeric",
            perform_staging=True)
        url = f5bigip.ssl.get_waf_entity_url(name="URL1",
            protocol="http")
        url2 = f5bigip.ssl.get_waf_entity_url(name="URL2")
        test_awaf = f5bigip.WafPolicy("test-awaf",
            name="testpolicyravi",
            partition="Common",
            template_name="POLICY_TEMPLATE_RAPID_DEPLOYMENT",
            application_language="utf-8",
            enforcement_mode="blocking",
            server_technologies=[
                "MySQL",
                "Unix/Linux",
                "MongoDB",
            ],
            parameters=[
                param1.json,
                param2.json,
            ],
            urls=[
                url.json,
                url2.json,
            ])
        ```

        ## Import

        An existing WAF Policy or if the WAF Policy has been manually created or modified on the BIG-IP WebUI, it can be imported using its `id`.

        e.g:

        ```sh
        $ pulumi import f5bigip:index/wafPolicy:WafPolicy example <id>
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[_builtins.str] application_language: The character encoding for the web application. The character encoding determines how the policy processes the character sets. The default is `utf-8`
        :param pulumi.Input[_builtins.bool] case_insensitive: Specifies whether the security policy treats microservice URLs, file types, URLs, and parameters as case sensitive or not. When this setting is enabled, the system stores these security policy elements in lowercase in the security policy configuration
        :param pulumi.Input[_builtins.str] description: Specifies the description of the policy.
        :param pulumi.Input[_builtins.bool] enable_passivemode: Passive Mode allows the policy to be associated with a Performance L4 Virtual Server (using a FastL4 profile). With FastL4, traffic is analyzed but is not modified in any way.
        :param pulumi.Input[_builtins.str] enforcement_mode: How the system processes a request that triggers a security policy violation
        :param pulumi.Input[Sequence[pulumi.Input[Union['WafPolicyFileTypeArgs', 'WafPolicyFileTypeArgsDict']]]] file_types: `file_types` takes list of file-types options to be used for policy builder.
               See file types below for more details.
        :param pulumi.Input[Sequence[pulumi.Input[Union['WafPolicyGraphqlProfileArgs', 'WafPolicyGraphqlProfileArgsDict']]]] graphql_profiles: `graphql_profiles` takes list of graphql profile options to be used for policy builder.
               See graphql profiles below for more details.
        :param pulumi.Input[Sequence[pulumi.Input[Union['WafPolicyHostNameArgs', 'WafPolicyHostNameArgsDict']]]] host_names: specify the list of host name that is used to access the application
        :param pulumi.Input[Sequence[pulumi.Input[Union['WafPolicyIpExceptionArgs', 'WafPolicyIpExceptionArgsDict']]]] ip_exceptions: `ip_exceptions` takes list of IP address exception,An IP address exception is an IP address that you want the system to treat in a specific way for a security policy.For example, you can specify IP addresses from which the system should always trust traffic.
               See IP Exceptions below for more details.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] modifications: the modifications section includes actions that modify the declarative policy as it is defined in the adjustments
               section. The modifications section is updated manually, with the changes generally driven by the learning suggestions
               provided by the BIG-IP.
        :param pulumi.Input[_builtins.str] name: The unique user-given name of the policy. Policy names cannot contain spaces or special characters. Allowed characters are a-z, A-Z, 0-9, dot, dash (-), colon (:) and underscore (_).
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] open_api_files: This section defines the Link for open api files on the policy.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] parameters: This section defines parameters that the security policy permits in requests.
        :param pulumi.Input[_builtins.str] partition: Specifies the partition of the policy. Default is `Common`
        :param pulumi.Input[Sequence[pulumi.Input[Union['WafPolicyPolicyBuilderArgs', 'WafPolicyPolicyBuilderArgsDict']]]] policy_builders: `policy_builder` block will provide `learning_mode` options to be used for policy builder.
               See policy builder below for more details.
        :param pulumi.Input[_builtins.str] policy_id: The id of the A.WAF Policy as it would be calculated on the BIG-IP.
        :param pulumi.Input[_builtins.str] policy_import_json: The payload of the WAF Policy to be used for IMPORT on to BIG-IP.
        :param pulumi.Input[_builtins.bool] protocol_independent: When creating a security policy, you can determine whether a security policy differentiates between HTTP and HTTPS URLs. If enabled, the security policy differentiates between HTTP and HTTPS URLs. If disabled, the security policy configures URLs without specifying a specific protocol. This is useful for applications that behave the same for HTTP and HTTPS, and it keeps the security policy from including the same URL twice.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] server_technologies: The server technology is a server-side application, framework, web server or operating system type that is configured in the policy in order to adapt the policy to the checks needed for the respective technology.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] signature_sets: Defines behavior when signatures found within a signature-set are detected in a request. Settings are culmulative, so if a signature is found in any set with block enabled, that signature will have block enabled.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] signatures: This section defines the properties of a signature on the policy.
        :param pulumi.Input[Sequence[pulumi.Input[Union['WafPolicySignaturesSettingArgs', 'WafPolicySignaturesSettingArgsDict']]]] signatures_settings: bulk signature setting
        :param pulumi.Input[_builtins.str] template_link: Specifies the Link of the template used for the policy creation.
        :param pulumi.Input[_builtins.str] template_name: Specifies the name of the template used for the policy creation.
        :param pulumi.Input[_builtins.str] type: The type of policy you want to create. The default policy type is `security`.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] urls: In a security policy, you can manually specify the HTTP URLs that are allowed (or disallowed) in traffic to the web application being protected. If you are using automatic policy building (and the policy includes learning URLs), the system can determine which URLs to add, based on legitimate traffic.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: WafPolicyArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        ## Example Usage

        ```python
        import pulumi
        import pulumi_f5bigip as f5bigip

        param1 = f5bigip.ssl.get_waf_entity_parameter(name="Param1",
            type="explicit",
            data_type="alpha-numeric",
            perform_staging=True)
        param2 = f5bigip.ssl.get_waf_entity_parameter(name="Param2",
            type="explicit",
            data_type="alpha-numeric",
            perform_staging=True)
        url = f5bigip.ssl.get_waf_entity_url(name="URL1",
            protocol="http")
        url2 = f5bigip.ssl.get_waf_entity_url(name="URL2")
        test_awaf = f5bigip.WafPolicy("test-awaf",
            name="testpolicyravi",
            partition="Common",
            template_name="POLICY_TEMPLATE_RAPID_DEPLOYMENT",
            application_language="utf-8",
            enforcement_mode="blocking",
            server_technologies=[
                "MySQL",
                "Unix/Linux",
                "MongoDB",
            ],
            parameters=[
                param1.json,
                param2.json,
            ],
            urls=[
                url.json,
                url2.json,
            ])
        ```

        ## Import

        An existing WAF Policy or if the WAF Policy has been manually created or modified on the BIG-IP WebUI, it can be imported using its `id`.

        e.g:

        ```sh
        $ pulumi import f5bigip:index/wafPolicy:WafPolicy example <id>
        ```

        :param str resource_name: The name of the resource.
        :param WafPolicyArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(WafPolicyArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 application_language: Optional[pulumi.Input[_builtins.str]] = None,
                 case_insensitive: Optional[pulumi.Input[_builtins.bool]] = None,
                 description: Optional[pulumi.Input[_builtins.str]] = None,
                 enable_passivemode: Optional[pulumi.Input[_builtins.bool]] = None,
                 enforcement_mode: Optional[pulumi.Input[_builtins.str]] = None,
                 file_types: Optional[pulumi.Input[Sequence[pulumi.Input[Union['WafPolicyFileTypeArgs', 'WafPolicyFileTypeArgsDict']]]]] = None,
                 graphql_profiles: Optional[pulumi.Input[Sequence[pulumi.Input[Union['WafPolicyGraphqlProfileArgs', 'WafPolicyGraphqlProfileArgsDict']]]]] = None,
                 host_names: Optional[pulumi.Input[Sequence[pulumi.Input[Union['WafPolicyHostNameArgs', 'WafPolicyHostNameArgsDict']]]]] = None,
                 ip_exceptions: Optional[pulumi.Input[Sequence[pulumi.Input[Union['WafPolicyIpExceptionArgs', 'WafPolicyIpExceptionArgsDict']]]]] = None,
                 modifications: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 name: Optional[pulumi.Input[_builtins.str]] = None,
                 open_api_files: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 parameters: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 partition: Optional[pulumi.Input[_builtins.str]] = None,
                 policy_builders: Optional[pulumi.Input[Sequence[pulumi.Input[Union['WafPolicyPolicyBuilderArgs', 'WafPolicyPolicyBuilderArgsDict']]]]] = None,
                 policy_id: Optional[pulumi.Input[_builtins.str]] = None,
                 policy_import_json: Optional[pulumi.Input[_builtins.str]] = None,
                 protocol_independent: Optional[pulumi.Input[_builtins.bool]] = None,
                 server_technologies: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 signature_sets: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 signatures: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 signatures_settings: Optional[pulumi.Input[Sequence[pulumi.Input[Union['WafPolicySignaturesSettingArgs', 'WafPolicySignaturesSettingArgsDict']]]]] = None,
                 template_link: Optional[pulumi.Input[_builtins.str]] = None,
                 template_name: Optional[pulumi.Input[_builtins.str]] = None,
                 type: Optional[pulumi.Input[_builtins.str]] = None,
                 urls: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = WafPolicyArgs.__new__(WafPolicyArgs)

            __props__.__dict__["application_language"] = application_language
            __props__.__dict__["case_insensitive"] = case_insensitive
            __props__.__dict__["description"] = description
            __props__.__dict__["enable_passivemode"] = enable_passivemode
            __props__.__dict__["enforcement_mode"] = enforcement_mode
            __props__.__dict__["file_types"] = file_types
            __props__.__dict__["graphql_profiles"] = graphql_profiles
            __props__.__dict__["host_names"] = host_names
            __props__.__dict__["ip_exceptions"] = ip_exceptions
            __props__.__dict__["modifications"] = modifications
            if name is None and not opts.urn:
                raise TypeError("Missing required property 'name'")
            __props__.__dict__["name"] = name
            __props__.__dict__["open_api_files"] = open_api_files
            __props__.__dict__["parameters"] = parameters
            __props__.__dict__["partition"] = partition
            __props__.__dict__["policy_builders"] = policy_builders
            __props__.__dict__["policy_id"] = policy_id
            __props__.__dict__["policy_import_json"] = policy_import_json
            __props__.__dict__["protocol_independent"] = protocol_independent
            __props__.__dict__["server_technologies"] = server_technologies
            __props__.__dict__["signature_sets"] = signature_sets
            __props__.__dict__["signatures"] = signatures
            __props__.__dict__["signatures_settings"] = signatures_settings
            __props__.__dict__["template_link"] = template_link
            if template_name is None and not opts.urn:
                raise TypeError("Missing required property 'template_name'")
            __props__.__dict__["template_name"] = template_name
            __props__.__dict__["type"] = type
            __props__.__dict__["urls"] = urls
            __props__.__dict__["policy_export_json"] = None
        super(WafPolicy, __self__).__init__(
            'f5bigip:index/wafPolicy:WafPolicy',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            application_language: Optional[pulumi.Input[_builtins.str]] = None,
            case_insensitive: Optional[pulumi.Input[_builtins.bool]] = None,
            description: Optional[pulumi.Input[_builtins.str]] = None,
            enable_passivemode: Optional[pulumi.Input[_builtins.bool]] = None,
            enforcement_mode: Optional[pulumi.Input[_builtins.str]] = None,
            file_types: Optional[pulumi.Input[Sequence[pulumi.Input[Union['WafPolicyFileTypeArgs', 'WafPolicyFileTypeArgsDict']]]]] = None,
            graphql_profiles: Optional[pulumi.Input[Sequence[pulumi.Input[Union['WafPolicyGraphqlProfileArgs', 'WafPolicyGraphqlProfileArgsDict']]]]] = None,
            host_names: Optional[pulumi.Input[Sequence[pulumi.Input[Union['WafPolicyHostNameArgs', 'WafPolicyHostNameArgsDict']]]]] = None,
            ip_exceptions: Optional[pulumi.Input[Sequence[pulumi.Input[Union['WafPolicyIpExceptionArgs', 'WafPolicyIpExceptionArgsDict']]]]] = None,
            modifications: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
            name: Optional[pulumi.Input[_builtins.str]] = None,
            open_api_files: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
            parameters: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
            partition: Optional[pulumi.Input[_builtins.str]] = None,
            policy_builders: Optional[pulumi.Input[Sequence[pulumi.Input[Union['WafPolicyPolicyBuilderArgs', 'WafPolicyPolicyBuilderArgsDict']]]]] = None,
            policy_export_json: Optional[pulumi.Input[_builtins.str]] = None,
            policy_id: Optional[pulumi.Input[_builtins.str]] = None,
            policy_import_json: Optional[pulumi.Input[_builtins.str]] = None,
            protocol_independent: Optional[pulumi.Input[_builtins.bool]] = None,
            server_technologies: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
            signature_sets: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
            signatures: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
            signatures_settings: Optional[pulumi.Input[Sequence[pulumi.Input[Union['WafPolicySignaturesSettingArgs', 'WafPolicySignaturesSettingArgsDict']]]]] = None,
            template_link: Optional[pulumi.Input[_builtins.str]] = None,
            template_name: Optional[pulumi.Input[_builtins.str]] = None,
            type: Optional[pulumi.Input[_builtins.str]] = None,
            urls: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None) -> 'WafPolicy':
        """
        Get an existing WafPolicy resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[_builtins.str] application_language: The character encoding for the web application. The character encoding determines how the policy processes the character sets. The default is `utf-8`
        :param pulumi.Input[_builtins.bool] case_insensitive: Specifies whether the security policy treats microservice URLs, file types, URLs, and parameters as case sensitive or not. When this setting is enabled, the system stores these security policy elements in lowercase in the security policy configuration
        :param pulumi.Input[_builtins.str] description: Specifies the description of the policy.
        :param pulumi.Input[_builtins.bool] enable_passivemode: Passive Mode allows the policy to be associated with a Performance L4 Virtual Server (using a FastL4 profile). With FastL4, traffic is analyzed but is not modified in any way.
        :param pulumi.Input[_builtins.str] enforcement_mode: How the system processes a request that triggers a security policy violation
        :param pulumi.Input[Sequence[pulumi.Input[Union['WafPolicyFileTypeArgs', 'WafPolicyFileTypeArgsDict']]]] file_types: `file_types` takes list of file-types options to be used for policy builder.
               See file types below for more details.
        :param pulumi.Input[Sequence[pulumi.Input[Union['WafPolicyGraphqlProfileArgs', 'WafPolicyGraphqlProfileArgsDict']]]] graphql_profiles: `graphql_profiles` takes list of graphql profile options to be used for policy builder.
               See graphql profiles below for more details.
        :param pulumi.Input[Sequence[pulumi.Input[Union['WafPolicyHostNameArgs', 'WafPolicyHostNameArgsDict']]]] host_names: specify the list of host name that is used to access the application
        :param pulumi.Input[Sequence[pulumi.Input[Union['WafPolicyIpExceptionArgs', 'WafPolicyIpExceptionArgsDict']]]] ip_exceptions: `ip_exceptions` takes list of IP address exception,An IP address exception is an IP address that you want the system to treat in a specific way for a security policy.For example, you can specify IP addresses from which the system should always trust traffic.
               See IP Exceptions below for more details.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] modifications: the modifications section includes actions that modify the declarative policy as it is defined in the adjustments
               section. The modifications section is updated manually, with the changes generally driven by the learning suggestions
               provided by the BIG-IP.
        :param pulumi.Input[_builtins.str] name: The unique user-given name of the policy. Policy names cannot contain spaces or special characters. Allowed characters are a-z, A-Z, 0-9, dot, dash (-), colon (:) and underscore (_).
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] open_api_files: This section defines the Link for open api files on the policy.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] parameters: This section defines parameters that the security policy permits in requests.
        :param pulumi.Input[_builtins.str] partition: Specifies the partition of the policy. Default is `Common`
        :param pulumi.Input[Sequence[pulumi.Input[Union['WafPolicyPolicyBuilderArgs', 'WafPolicyPolicyBuilderArgsDict']]]] policy_builders: `policy_builder` block will provide `learning_mode` options to be used for policy builder.
               See policy builder below for more details.
        :param pulumi.Input[_builtins.str] policy_export_json: Exported WAF policy deployed on BIGIP.
        :param pulumi.Input[_builtins.str] policy_id: The id of the A.WAF Policy as it would be calculated on the BIG-IP.
        :param pulumi.Input[_builtins.str] policy_import_json: The payload of the WAF Policy to be used for IMPORT on to BIG-IP.
        :param pulumi.Input[_builtins.bool] protocol_independent: When creating a security policy, you can determine whether a security policy differentiates between HTTP and HTTPS URLs. If enabled, the security policy differentiates between HTTP and HTTPS URLs. If disabled, the security policy configures URLs without specifying a specific protocol. This is useful for applications that behave the same for HTTP and HTTPS, and it keeps the security policy from including the same URL twice.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] server_technologies: The server technology is a server-side application, framework, web server or operating system type that is configured in the policy in order to adapt the policy to the checks needed for the respective technology.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] signature_sets: Defines behavior when signatures found within a signature-set are detected in a request. Settings are culmulative, so if a signature is found in any set with block enabled, that signature will have block enabled.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] signatures: This section defines the properties of a signature on the policy.
        :param pulumi.Input[Sequence[pulumi.Input[Union['WafPolicySignaturesSettingArgs', 'WafPolicySignaturesSettingArgsDict']]]] signatures_settings: bulk signature setting
        :param pulumi.Input[_builtins.str] template_link: Specifies the Link of the template used for the policy creation.
        :param pulumi.Input[_builtins.str] template_name: Specifies the name of the template used for the policy creation.
        :param pulumi.Input[_builtins.str] type: The type of policy you want to create. The default policy type is `security`.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] urls: In a security policy, you can manually specify the HTTP URLs that are allowed (or disallowed) in traffic to the web application being protected. If you are using automatic policy building (and the policy includes learning URLs), the system can determine which URLs to add, based on legitimate traffic.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _WafPolicyState.__new__(_WafPolicyState)

        __props__.__dict__["application_language"] = application_language
        __props__.__dict__["case_insensitive"] = case_insensitive
        __props__.__dict__["description"] = description
        __props__.__dict__["enable_passivemode"] = enable_passivemode
        __props__.__dict__["enforcement_mode"] = enforcement_mode
        __props__.__dict__["file_types"] = file_types
        __props__.__dict__["graphql_profiles"] = graphql_profiles
        __props__.__dict__["host_names"] = host_names
        __props__.__dict__["ip_exceptions"] = ip_exceptions
        __props__.__dict__["modifications"] = modifications
        __props__.__dict__["name"] = name
        __props__.__dict__["open_api_files"] = open_api_files
        __props__.__dict__["parameters"] = parameters
        __props__.__dict__["partition"] = partition
        __props__.__dict__["policy_builders"] = policy_builders
        __props__.__dict__["policy_export_json"] = policy_export_json
        __props__.__dict__["policy_id"] = policy_id
        __props__.__dict__["policy_import_json"] = policy_import_json
        __props__.__dict__["protocol_independent"] = protocol_independent
        __props__.__dict__["server_technologies"] = server_technologies
        __props__.__dict__["signature_sets"] = signature_sets
        __props__.__dict__["signatures"] = signatures
        __props__.__dict__["signatures_settings"] = signatures_settings
        __props__.__dict__["template_link"] = template_link
        __props__.__dict__["template_name"] = template_name
        __props__.__dict__["type"] = type
        __props__.__dict__["urls"] = urls
        return WafPolicy(resource_name, opts=opts, __props__=__props__)

    @_builtins.property
    @pulumi.getter(name="applicationLanguage")
    def application_language(self) -> pulumi.Output[Optional[_builtins.str]]:
        """
        The character encoding for the web application. The character encoding determines how the policy processes the character sets. The default is `utf-8`
        """
        return pulumi.get(self, "application_language")

    @_builtins.property
    @pulumi.getter(name="caseInsensitive")
    def case_insensitive(self) -> pulumi.Output[Optional[_builtins.bool]]:
        """
        Specifies whether the security policy treats microservice URLs, file types, URLs, and parameters as case sensitive or not. When this setting is enabled, the system stores these security policy elements in lowercase in the security policy configuration
        """
        return pulumi.get(self, "case_insensitive")

    @_builtins.property
    @pulumi.getter
    def description(self) -> pulumi.Output[_builtins.str]:
        """
        Specifies the description of the policy.
        """
        return pulumi.get(self, "description")

    @_builtins.property
    @pulumi.getter(name="enablePassivemode")
    def enable_passivemode(self) -> pulumi.Output[Optional[_builtins.bool]]:
        """
        Passive Mode allows the policy to be associated with a Performance L4 Virtual Server (using a FastL4 profile). With FastL4, traffic is analyzed but is not modified in any way.
        """
        return pulumi.get(self, "enable_passivemode")

    @_builtins.property
    @pulumi.getter(name="enforcementMode")
    def enforcement_mode(self) -> pulumi.Output[Optional[_builtins.str]]:
        """
        How the system processes a request that triggers a security policy violation
        """
        return pulumi.get(self, "enforcement_mode")

    @_builtins.property
    @pulumi.getter(name="fileTypes")
    def file_types(self) -> pulumi.Output[Optional[Sequence['outputs.WafPolicyFileType']]]:
        """
        `file_types` takes list of file-types options to be used for policy builder.
        See file types below for more details.
        """
        return pulumi.get(self, "file_types")

    @_builtins.property
    @pulumi.getter(name="graphqlProfiles")
    def graphql_profiles(self) -> pulumi.Output[Optional[Sequence['outputs.WafPolicyGraphqlProfile']]]:
        """
        `graphql_profiles` takes list of graphql profile options to be used for policy builder.
        See graphql profiles below for more details.
        """
        return pulumi.get(self, "graphql_profiles")

    @_builtins.property
    @pulumi.getter(name="hostNames")
    def host_names(self) -> pulumi.Output[Optional[Sequence['outputs.WafPolicyHostName']]]:
        """
        specify the list of host name that is used to access the application
        """
        return pulumi.get(self, "host_names")

    @_builtins.property
    @pulumi.getter(name="ipExceptions")
    def ip_exceptions(self) -> pulumi.Output[Optional[Sequence['outputs.WafPolicyIpException']]]:
        """
        `ip_exceptions` takes list of IP address exception,An IP address exception is an IP address that you want the system to treat in a specific way for a security policy.For example, you can specify IP addresses from which the system should always trust traffic.
        See IP Exceptions below for more details.
        """
        return pulumi.get(self, "ip_exceptions")

    @_builtins.property
    @pulumi.getter
    def modifications(self) -> pulumi.Output[Optional[Sequence[_builtins.str]]]:
        """
        the modifications section includes actions that modify the declarative policy as it is defined in the adjustments
        section. The modifications section is updated manually, with the changes generally driven by the learning suggestions
        provided by the BIG-IP.
        """
        return pulumi.get(self, "modifications")

    @_builtins.property
    @pulumi.getter
    def name(self) -> pulumi.Output[_builtins.str]:
        """
        The unique user-given name of the policy. Policy names cannot contain spaces or special characters. Allowed characters are a-z, A-Z, 0-9, dot, dash (-), colon (:) and underscore (_).
        """
        return pulumi.get(self, "name")

    @_builtins.property
    @pulumi.getter(name="openApiFiles")
    def open_api_files(self) -> pulumi.Output[Optional[Sequence[_builtins.str]]]:
        """
        This section defines the Link for open api files on the policy.
        """
        return pulumi.get(self, "open_api_files")

    @_builtins.property
    @pulumi.getter
    def parameters(self) -> pulumi.Output[Optional[Sequence[_builtins.str]]]:
        """
        This section defines parameters that the security policy permits in requests.
        """
        return pulumi.get(self, "parameters")

    @_builtins.property
    @pulumi.getter
    def partition(self) -> pulumi.Output[Optional[_builtins.str]]:
        """
        Specifies the partition of the policy. Default is `Common`
        """
        return pulumi.get(self, "partition")

    @_builtins.property
    @pulumi.getter(name="policyBuilders")
    def policy_builders(self) -> pulumi.Output[Optional[Sequence['outputs.WafPolicyPolicyBuilder']]]:
        """
        `policy_builder` block will provide `learning_mode` options to be used for policy builder.
        See policy builder below for more details.
        """
        return pulumi.get(self, "policy_builders")

    @_builtins.property
    @pulumi.getter(name="policyExportJson")
    def policy_export_json(self) -> pulumi.Output[_builtins.str]:
        """
        Exported WAF policy deployed on BIGIP.
        """
        return pulumi.get(self, "policy_export_json")

    @_builtins.property
    @pulumi.getter(name="policyId")
    def policy_id(self) -> pulumi.Output[_builtins.str]:
        """
        The id of the A.WAF Policy as it would be calculated on the BIG-IP.
        """
        return pulumi.get(self, "policy_id")

    @_builtins.property
    @pulumi.getter(name="policyImportJson")
    def policy_import_json(self) -> pulumi.Output[Optional[_builtins.str]]:
        """
        The payload of the WAF Policy to be used for IMPORT on to BIG-IP.
        """
        return pulumi.get(self, "policy_import_json")

    @_builtins.property
    @pulumi.getter(name="protocolIndependent")
    def protocol_independent(self) -> pulumi.Output[Optional[_builtins.bool]]:
        """
        When creating a security policy, you can determine whether a security policy differentiates between HTTP and HTTPS URLs. If enabled, the security policy differentiates between HTTP and HTTPS URLs. If disabled, the security policy configures URLs without specifying a specific protocol. This is useful for applications that behave the same for HTTP and HTTPS, and it keeps the security policy from including the same URL twice.
        """
        return pulumi.get(self, "protocol_independent")

    @_builtins.property
    @pulumi.getter(name="serverTechnologies")
    def server_technologies(self) -> pulumi.Output[Optional[Sequence[_builtins.str]]]:
        """
        The server technology is a server-side application, framework, web server or operating system type that is configured in the policy in order to adapt the policy to the checks needed for the respective technology.
        """
        return pulumi.get(self, "server_technologies")

    @_builtins.property
    @pulumi.getter(name="signatureSets")
    def signature_sets(self) -> pulumi.Output[Optional[Sequence[_builtins.str]]]:
        """
        Defines behavior when signatures found within a signature-set are detected in a request. Settings are culmulative, so if a signature is found in any set with block enabled, that signature will have block enabled.
        """
        return pulumi.get(self, "signature_sets")

    @_builtins.property
    @pulumi.getter
    def signatures(self) -> pulumi.Output[Optional[Sequence[_builtins.str]]]:
        """
        This section defines the properties of a signature on the policy.
        """
        return pulumi.get(self, "signatures")

    @_builtins.property
    @pulumi.getter(name="signaturesSettings")
    def signatures_settings(self) -> pulumi.Output[Optional[Sequence['outputs.WafPolicySignaturesSetting']]]:
        """
        bulk signature setting
        """
        return pulumi.get(self, "signatures_settings")

    @_builtins.property
    @pulumi.getter(name="templateLink")
    def template_link(self) -> pulumi.Output[Optional[_builtins.str]]:
        """
        Specifies the Link of the template used for the policy creation.
        """
        return pulumi.get(self, "template_link")

    @_builtins.property
    @pulumi.getter(name="templateName")
    def template_name(self) -> pulumi.Output[_builtins.str]:
        """
        Specifies the name of the template used for the policy creation.
        """
        return pulumi.get(self, "template_name")

    @_builtins.property
    @pulumi.getter
    def type(self) -> pulumi.Output[Optional[_builtins.str]]:
        """
        The type of policy you want to create. The default policy type is `security`.
        """
        return pulumi.get(self, "type")

    @_builtins.property
    @pulumi.getter
    def urls(self) -> pulumi.Output[Optional[Sequence[_builtins.str]]]:
        """
        In a security policy, you can manually specify the HTTP URLs that are allowed (or disallowed) in traffic to the web application being protected. If you are using automatic policy building (and the policy includes learning URLs), the system can determine which URLs to add, based on legitimate traffic.
        """
        return pulumi.get(self, "urls")




# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from .. import _utilities

__all__ = ['PoolAttachmentArgs', 'PoolAttachment']

@pulumi.input_type
class PoolAttachmentArgs:
    def __init__(__self__, *,
                 node: pulumi.Input[_builtins.str],
                 pool: pulumi.Input[_builtins.str],
                 connection_limit: Optional[pulumi.Input[_builtins.int]] = None,
                 connection_rate_limit: Optional[pulumi.Input[_builtins.str]] = None,
                 dynamic_ratio: Optional[pulumi.Input[_builtins.int]] = None,
                 fqdn_autopopulate: Optional[pulumi.Input[_builtins.str]] = None,
                 monitor: Optional[pulumi.Input[_builtins.str]] = None,
                 priority_group: Optional[pulumi.Input[_builtins.int]] = None,
                 ratio: Optional[pulumi.Input[_builtins.int]] = None,
                 state: Optional[pulumi.Input[_builtins.str]] = None):
        """
        The set of arguments for constructing a PoolAttachment resource.
        :param pulumi.Input[_builtins.str] node: Pool member address/fqdn with service port, (ex: `1.1.1.1:80/www.google.com:80`). (Note: Member will be in same partition of Pool)
        :param pulumi.Input[_builtins.str] pool: Name of the pool to which members should be attached,it should be "full path".The full path is the combination of the partition + name of the pool.(For example `/Common/my-pool`) or partition + directory + name of the pool (For example `/Common/test/my-pool`).When including directory in fullpath we have to make sure it is created in the given partition before using it.
        :param pulumi.Input[_builtins.int] connection_limit: Specifies a maximum established connection limit for a pool member or node.The default is 0, meaning that there is no limit to the number of connections.
        :param pulumi.Input[_builtins.str] connection_rate_limit: Specifies the maximum number of connections-per-second allowed for a pool member,The default is 0.
        :param pulumi.Input[_builtins.int] dynamic_ratio: Specifies the fixed ratio value used for a node during ratio load balancing.
        :param pulumi.Input[_builtins.str] fqdn_autopopulate: Specifies whether the system automatically creates ephemeral nodes using the IP addresses returned by the resolution of a DNS query for a node defined by an FQDN. The default is enabled
        :param pulumi.Input[_builtins.str] monitor: Specifies the health monitors that the system uses to monitor this pool member,value can be `none` (or) `default` (or) list of monitors joined with and ( ex: `/Common/test_monitor_pa_tc1 and /Common/gateway_icmp`).
        :param pulumi.Input[_builtins.int] priority_group: Specifies a number representing the priority group for the pool member. The default is 0, meaning that the member has no priority
        :param pulumi.Input[_builtins.int] ratio: "Specifies the ratio weight to assign to the pool member. Valid values range from 1 through 65535. The default is 1, which means that each pool member has an equal ratio proportion.".
        :param pulumi.Input[_builtins.str] state: Specifies the state the pool member should be in,value can be `enabled` (or) `disabled` (or) `forced_offline`).
        """
        pulumi.set(__self__, "node", node)
        pulumi.set(__self__, "pool", pool)
        if connection_limit is not None:
            pulumi.set(__self__, "connection_limit", connection_limit)
        if connection_rate_limit is not None:
            pulumi.set(__self__, "connection_rate_limit", connection_rate_limit)
        if dynamic_ratio is not None:
            pulumi.set(__self__, "dynamic_ratio", dynamic_ratio)
        if fqdn_autopopulate is not None:
            pulumi.set(__self__, "fqdn_autopopulate", fqdn_autopopulate)
        if monitor is not None:
            pulumi.set(__self__, "monitor", monitor)
        if priority_group is not None:
            pulumi.set(__self__, "priority_group", priority_group)
        if ratio is not None:
            pulumi.set(__self__, "ratio", ratio)
        if state is not None:
            pulumi.set(__self__, "state", state)

    @_builtins.property
    @pulumi.getter
    def node(self) -> pulumi.Input[_builtins.str]:
        """
        Pool member address/fqdn with service port, (ex: `1.1.1.1:80/www.google.com:80`). (Note: Member will be in same partition of Pool)
        """
        return pulumi.get(self, "node")

    @node.setter
    def node(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "node", value)

    @_builtins.property
    @pulumi.getter
    def pool(self) -> pulumi.Input[_builtins.str]:
        """
        Name of the pool to which members should be attached,it should be "full path".The full path is the combination of the partition + name of the pool.(For example `/Common/my-pool`) or partition + directory + name of the pool (For example `/Common/test/my-pool`).When including directory in fullpath we have to make sure it is created in the given partition before using it.
        """
        return pulumi.get(self, "pool")

    @pool.setter
    def pool(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "pool", value)

    @_builtins.property
    @pulumi.getter(name="connectionLimit")
    def connection_limit(self) -> Optional[pulumi.Input[_builtins.int]]:
        """
        Specifies a maximum established connection limit for a pool member or node.The default is 0, meaning that there is no limit to the number of connections.
        """
        return pulumi.get(self, "connection_limit")

    @connection_limit.setter
    def connection_limit(self, value: Optional[pulumi.Input[_builtins.int]]):
        pulumi.set(self, "connection_limit", value)

    @_builtins.property
    @pulumi.getter(name="connectionRateLimit")
    def connection_rate_limit(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Specifies the maximum number of connections-per-second allowed for a pool member,The default is 0.
        """
        return pulumi.get(self, "connection_rate_limit")

    @connection_rate_limit.setter
    def connection_rate_limit(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "connection_rate_limit", value)

    @_builtins.property
    @pulumi.getter(name="dynamicRatio")
    def dynamic_ratio(self) -> Optional[pulumi.Input[_builtins.int]]:
        """
        Specifies the fixed ratio value used for a node during ratio load balancing.
        """
        return pulumi.get(self, "dynamic_ratio")

    @dynamic_ratio.setter
    def dynamic_ratio(self, value: Optional[pulumi.Input[_builtins.int]]):
        pulumi.set(self, "dynamic_ratio", value)

    @_builtins.property
    @pulumi.getter(name="fqdnAutopopulate")
    def fqdn_autopopulate(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Specifies whether the system automatically creates ephemeral nodes using the IP addresses returned by the resolution of a DNS query for a node defined by an FQDN. The default is enabled
        """
        return pulumi.get(self, "fqdn_autopopulate")

    @fqdn_autopopulate.setter
    def fqdn_autopopulate(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "fqdn_autopopulate", value)

    @_builtins.property
    @pulumi.getter
    def monitor(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Specifies the health monitors that the system uses to monitor this pool member,value can be `none` (or) `default` (or) list of monitors joined with and ( ex: `/Common/test_monitor_pa_tc1 and /Common/gateway_icmp`).
        """
        return pulumi.get(self, "monitor")

    @monitor.setter
    def monitor(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "monitor", value)

    @_builtins.property
    @pulumi.getter(name="priorityGroup")
    def priority_group(self) -> Optional[pulumi.Input[_builtins.int]]:
        """
        Specifies a number representing the priority group for the pool member. The default is 0, meaning that the member has no priority
        """
        return pulumi.get(self, "priority_group")

    @priority_group.setter
    def priority_group(self, value: Optional[pulumi.Input[_builtins.int]]):
        pulumi.set(self, "priority_group", value)

    @_builtins.property
    @pulumi.getter
    def ratio(self) -> Optional[pulumi.Input[_builtins.int]]:
        """
        "Specifies the ratio weight to assign to the pool member. Valid values range from 1 through 65535. The default is 1, which means that each pool member has an equal ratio proportion.".
        """
        return pulumi.get(self, "ratio")

    @ratio.setter
    def ratio(self, value: Optional[pulumi.Input[_builtins.int]]):
        pulumi.set(self, "ratio", value)

    @_builtins.property
    @pulumi.getter
    def state(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Specifies the state the pool member should be in,value can be `enabled` (or) `disabled` (or) `forced_offline`).
        """
        return pulumi.get(self, "state")

    @state.setter
    def state(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "state", value)


@pulumi.input_type
class _PoolAttachmentState:
    def __init__(__self__, *,
                 connection_limit: Optional[pulumi.Input[_builtins.int]] = None,
                 connection_rate_limit: Optional[pulumi.Input[_builtins.str]] = None,
                 dynamic_ratio: Optional[pulumi.Input[_builtins.int]] = None,
                 fqdn_autopopulate: Optional[pulumi.Input[_builtins.str]] = None,
                 monitor: Optional[pulumi.Input[_builtins.str]] = None,
                 node: Optional[pulumi.Input[_builtins.str]] = None,
                 pool: Optional[pulumi.Input[_builtins.str]] = None,
                 priority_group: Optional[pulumi.Input[_builtins.int]] = None,
                 ratio: Optional[pulumi.Input[_builtins.int]] = None,
                 state: Optional[pulumi.Input[_builtins.str]] = None):
        """
        Input properties used for looking up and filtering PoolAttachment resources.
        :param pulumi.Input[_builtins.int] connection_limit: Specifies a maximum established connection limit for a pool member or node.The default is 0, meaning that there is no limit to the number of connections.
        :param pulumi.Input[_builtins.str] connection_rate_limit: Specifies the maximum number of connections-per-second allowed for a pool member,The default is 0.
        :param pulumi.Input[_builtins.int] dynamic_ratio: Specifies the fixed ratio value used for a node during ratio load balancing.
        :param pulumi.Input[_builtins.str] fqdn_autopopulate: Specifies whether the system automatically creates ephemeral nodes using the IP addresses returned by the resolution of a DNS query for a node defined by an FQDN. The default is enabled
        :param pulumi.Input[_builtins.str] monitor: Specifies the health monitors that the system uses to monitor this pool member,value can be `none` (or) `default` (or) list of monitors joined with and ( ex: `/Common/test_monitor_pa_tc1 and /Common/gateway_icmp`).
        :param pulumi.Input[_builtins.str] node: Pool member address/fqdn with service port, (ex: `1.1.1.1:80/www.google.com:80`). (Note: Member will be in same partition of Pool)
        :param pulumi.Input[_builtins.str] pool: Name of the pool to which members should be attached,it should be "full path".The full path is the combination of the partition + name of the pool.(For example `/Common/my-pool`) or partition + directory + name of the pool (For example `/Common/test/my-pool`).When including directory in fullpath we have to make sure it is created in the given partition before using it.
        :param pulumi.Input[_builtins.int] priority_group: Specifies a number representing the priority group for the pool member. The default is 0, meaning that the member has no priority
        :param pulumi.Input[_builtins.int] ratio: "Specifies the ratio weight to assign to the pool member. Valid values range from 1 through 65535. The default is 1, which means that each pool member has an equal ratio proportion.".
        :param pulumi.Input[_builtins.str] state: Specifies the state the pool member should be in,value can be `enabled` (or) `disabled` (or) `forced_offline`).
        """
        if connection_limit is not None:
            pulumi.set(__self__, "connection_limit", connection_limit)
        if connection_rate_limit is not None:
            pulumi.set(__self__, "connection_rate_limit", connection_rate_limit)
        if dynamic_ratio is not None:
            pulumi.set(__self__, "dynamic_ratio", dynamic_ratio)
        if fqdn_autopopulate is not None:
            pulumi.set(__self__, "fqdn_autopopulate", fqdn_autopopulate)
        if monitor is not None:
            pulumi.set(__self__, "monitor", monitor)
        if node is not None:
            pulumi.set(__self__, "node", node)
        if pool is not None:
            pulumi.set(__self__, "pool", pool)
        if priority_group is not None:
            pulumi.set(__self__, "priority_group", priority_group)
        if ratio is not None:
            pulumi.set(__self__, "ratio", ratio)
        if state is not None:
            pulumi.set(__self__, "state", state)

    @_builtins.property
    @pulumi.getter(name="connectionLimit")
    def connection_limit(self) -> Optional[pulumi.Input[_builtins.int]]:
        """
        Specifies a maximum established connection limit for a pool member or node.The default is 0, meaning that there is no limit to the number of connections.
        """
        return pulumi.get(self, "connection_limit")

    @connection_limit.setter
    def connection_limit(self, value: Optional[pulumi.Input[_builtins.int]]):
        pulumi.set(self, "connection_limit", value)

    @_builtins.property
    @pulumi.getter(name="connectionRateLimit")
    def connection_rate_limit(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Specifies the maximum number of connections-per-second allowed for a pool member,The default is 0.
        """
        return pulumi.get(self, "connection_rate_limit")

    @connection_rate_limit.setter
    def connection_rate_limit(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "connection_rate_limit", value)

    @_builtins.property
    @pulumi.getter(name="dynamicRatio")
    def dynamic_ratio(self) -> Optional[pulumi.Input[_builtins.int]]:
        """
        Specifies the fixed ratio value used for a node during ratio load balancing.
        """
        return pulumi.get(self, "dynamic_ratio")

    @dynamic_ratio.setter
    def dynamic_ratio(self, value: Optional[pulumi.Input[_builtins.int]]):
        pulumi.set(self, "dynamic_ratio", value)

    @_builtins.property
    @pulumi.getter(name="fqdnAutopopulate")
    def fqdn_autopopulate(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Specifies whether the system automatically creates ephemeral nodes using the IP addresses returned by the resolution of a DNS query for a node defined by an FQDN. The default is enabled
        """
        return pulumi.get(self, "fqdn_autopopulate")

    @fqdn_autopopulate.setter
    def fqdn_autopopulate(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "fqdn_autopopulate", value)

    @_builtins.property
    @pulumi.getter
    def monitor(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Specifies the health monitors that the system uses to monitor this pool member,value can be `none` (or) `default` (or) list of monitors joined with and ( ex: `/Common/test_monitor_pa_tc1 and /Common/gateway_icmp`).
        """
        return pulumi.get(self, "monitor")

    @monitor.setter
    def monitor(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "monitor", value)

    @_builtins.property
    @pulumi.getter
    def node(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Pool member address/fqdn with service port, (ex: `1.1.1.1:80/www.google.com:80`). (Note: Member will be in same partition of Pool)
        """
        return pulumi.get(self, "node")

    @node.setter
    def node(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "node", value)

    @_builtins.property
    @pulumi.getter
    def pool(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Name of the pool to which members should be attached,it should be "full path".The full path is the combination of the partition + name of the pool.(For example `/Common/my-pool`) or partition + directory + name of the pool (For example `/Common/test/my-pool`).When including directory in fullpath we have to make sure it is created in the given partition before using it.
        """
        return pulumi.get(self, "pool")

    @pool.setter
    def pool(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "pool", value)

    @_builtins.property
    @pulumi.getter(name="priorityGroup")
    def priority_group(self) -> Optional[pulumi.Input[_builtins.int]]:
        """
        Specifies a number representing the priority group for the pool member. The default is 0, meaning that the member has no priority
        """
        return pulumi.get(self, "priority_group")

    @priority_group.setter
    def priority_group(self, value: Optional[pulumi.Input[_builtins.int]]):
        pulumi.set(self, "priority_group", value)

    @_builtins.property
    @pulumi.getter
    def ratio(self) -> Optional[pulumi.Input[_builtins.int]]:
        """
        "Specifies the ratio weight to assign to the pool member. Valid values range from 1 through 65535. The default is 1, which means that each pool member has an equal ratio proportion.".
        """
        return pulumi.get(self, "ratio")

    @ratio.setter
    def ratio(self, value: Optional[pulumi.Input[_builtins.int]]):
        pulumi.set(self, "ratio", value)

    @_builtins.property
    @pulumi.getter
    def state(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Specifies the state the pool member should be in,value can be `enabled` (or) `disabled` (or) `forced_offline`).
        """
        return pulumi.get(self, "state")

    @state.setter
    def state(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "state", value)


@pulumi.type_token("f5bigip:ltm/poolAttachment:PoolAttachment")
class PoolAttachment(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 connection_limit: Optional[pulumi.Input[_builtins.int]] = None,
                 connection_rate_limit: Optional[pulumi.Input[_builtins.str]] = None,
                 dynamic_ratio: Optional[pulumi.Input[_builtins.int]] = None,
                 fqdn_autopopulate: Optional[pulumi.Input[_builtins.str]] = None,
                 monitor: Optional[pulumi.Input[_builtins.str]] = None,
                 node: Optional[pulumi.Input[_builtins.str]] = None,
                 pool: Optional[pulumi.Input[_builtins.str]] = None,
                 priority_group: Optional[pulumi.Input[_builtins.int]] = None,
                 ratio: Optional[pulumi.Input[_builtins.int]] = None,
                 state: Optional[pulumi.Input[_builtins.str]] = None,
                 __props__=None):
        """
        `ltm.PoolAttachment` Manages nodes membership in pools

        ## Example Usage

        There are two ways to use `ltm.PoolAttachment` resource for `node` attribute

        * It can be reference from `ltm.Node` (or)
        * It can be specify directly with `ipv4:port`/`fqdn:port`/`ipv6.port` which will also create node and attach member to pool.

        > For adding IPv6 node/member to pool it should be specific in `node` attribute in format like `ipv6_address.port`.
        IPv4 should be specified as `ipv4_address:port`

        ### Usage Pool attachment with node/member directly attaching to pool.

        node can be specified in format `ipv4:port` / `fqdn:port` / `ipv6.port`

        ```python
        import pulumi
        import pulumi_f5bigip as f5bigip

        monitor = f5bigip.ltm.Monitor("monitor",
            name="/Common/terraform_monitor",
            parent="/Common/http",
            send="GET /some/path\\x0d\\n",
            timeout=999,
            interval=998)
        pool = f5bigip.ltm.Pool("pool",
            name="/Common/terraform-pool",
            load_balancing_mode="round-robin",
            monitors=[monitor.name],
            allow_snat="yes",
            allow_nat="yes")
        # attaching ipv4 address with service port
        ipv4_node_attach = f5bigip.ltm.PoolAttachment("ipv4_node_attach",
            pool=pool.name,
            node="1.1.1.1:80")
        # attaching ipv6 address with service port
        ipv6_node_attach = f5bigip.ltm.PoolAttachment("ipv6_node_attach",
            pool=pool.name,
            node="2003::4.80")
        ```

        ### Usage Pool attachment with node referenced from `ltm.Node`

        ```python
        import pulumi
        import pulumi_f5bigip as f5bigip

        monitor = f5bigip.ltm.Monitor("monitor",
            name="/Common/terraform_monitor",
            parent="/Common/http",
            send="GET /some/path\\x0d\\n",
            timeout=999,
            interval=998)
        pool = f5bigip.ltm.Pool("pool",
            name="/Common/terraform-pool",
            load_balancing_mode="round-robin",
            monitors=[monitor.name],
            allow_snat="yes",
            allow_nat="yes")
        node = f5bigip.ltm.Node("node",
            name="/Common/terraform_node",
            address="192.168.30.2")
        attach_node = f5bigip.ltm.PoolAttachment("attach_node",
            pool=pool.name,
            node=node.name.apply(lambda name: f"{name}:80"))
        ```

        ## Importing

        An existing pool attachment (i.e. pool membership) can be imported into this resource by supplying both the pool full path, and the node full path with the relevant port. If the pool or node membership is not found, an error will be returned. An example is below:

        ```sh
        $ terraform import bigip_ltm_pool_attachment.node-pool-attach \\
        	'{"pool": "/Common/terraform-pool", "node": "/Common/node1:80"}'
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[_builtins.int] connection_limit: Specifies a maximum established connection limit for a pool member or node.The default is 0, meaning that there is no limit to the number of connections.
        :param pulumi.Input[_builtins.str] connection_rate_limit: Specifies the maximum number of connections-per-second allowed for a pool member,The default is 0.
        :param pulumi.Input[_builtins.int] dynamic_ratio: Specifies the fixed ratio value used for a node during ratio load balancing.
        :param pulumi.Input[_builtins.str] fqdn_autopopulate: Specifies whether the system automatically creates ephemeral nodes using the IP addresses returned by the resolution of a DNS query for a node defined by an FQDN. The default is enabled
        :param pulumi.Input[_builtins.str] monitor: Specifies the health monitors that the system uses to monitor this pool member,value can be `none` (or) `default` (or) list of monitors joined with and ( ex: `/Common/test_monitor_pa_tc1 and /Common/gateway_icmp`).
        :param pulumi.Input[_builtins.str] node: Pool member address/fqdn with service port, (ex: `1.1.1.1:80/www.google.com:80`). (Note: Member will be in same partition of Pool)
        :param pulumi.Input[_builtins.str] pool: Name of the pool to which members should be attached,it should be "full path".The full path is the combination of the partition + name of the pool.(For example `/Common/my-pool`) or partition + directory + name of the pool (For example `/Common/test/my-pool`).When including directory in fullpath we have to make sure it is created in the given partition before using it.
        :param pulumi.Input[_builtins.int] priority_group: Specifies a number representing the priority group for the pool member. The default is 0, meaning that the member has no priority
        :param pulumi.Input[_builtins.int] ratio: "Specifies the ratio weight to assign to the pool member. Valid values range from 1 through 65535. The default is 1, which means that each pool member has an equal ratio proportion.".
        :param pulumi.Input[_builtins.str] state: Specifies the state the pool member should be in,value can be `enabled` (or) `disabled` (or) `forced_offline`).
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: PoolAttachmentArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        `ltm.PoolAttachment` Manages nodes membership in pools

        ## Example Usage

        There are two ways to use `ltm.PoolAttachment` resource for `node` attribute

        * It can be reference from `ltm.Node` (or)
        * It can be specify directly with `ipv4:port`/`fqdn:port`/`ipv6.port` which will also create node and attach member to pool.

        > For adding IPv6 node/member to pool it should be specific in `node` attribute in format like `ipv6_address.port`.
        IPv4 should be specified as `ipv4_address:port`

        ### Usage Pool attachment with node/member directly attaching to pool.

        node can be specified in format `ipv4:port` / `fqdn:port` / `ipv6.port`

        ```python
        import pulumi
        import pulumi_f5bigip as f5bigip

        monitor = f5bigip.ltm.Monitor("monitor",
            name="/Common/terraform_monitor",
            parent="/Common/http",
            send="GET /some/path\\x0d\\n",
            timeout=999,
            interval=998)
        pool = f5bigip.ltm.Pool("pool",
            name="/Common/terraform-pool",
            load_balancing_mode="round-robin",
            monitors=[monitor.name],
            allow_snat="yes",
            allow_nat="yes")
        # attaching ipv4 address with service port
        ipv4_node_attach = f5bigip.ltm.PoolAttachment("ipv4_node_attach",
            pool=pool.name,
            node="1.1.1.1:80")
        # attaching ipv6 address with service port
        ipv6_node_attach = f5bigip.ltm.PoolAttachment("ipv6_node_attach",
            pool=pool.name,
            node="2003::4.80")
        ```

        ### Usage Pool attachment with node referenced from `ltm.Node`

        ```python
        import pulumi
        import pulumi_f5bigip as f5bigip

        monitor = f5bigip.ltm.Monitor("monitor",
            name="/Common/terraform_monitor",
            parent="/Common/http",
            send="GET /some/path\\x0d\\n",
            timeout=999,
            interval=998)
        pool = f5bigip.ltm.Pool("pool",
            name="/Common/terraform-pool",
            load_balancing_mode="round-robin",
            monitors=[monitor.name],
            allow_snat="yes",
            allow_nat="yes")
        node = f5bigip.ltm.Node("node",
            name="/Common/terraform_node",
            address="192.168.30.2")
        attach_node = f5bigip.ltm.PoolAttachment("attach_node",
            pool=pool.name,
            node=node.name.apply(lambda name: f"{name}:80"))
        ```

        ## Importing

        An existing pool attachment (i.e. pool membership) can be imported into this resource by supplying both the pool full path, and the node full path with the relevant port. If the pool or node membership is not found, an error will be returned. An example is below:

        ```sh
        $ terraform import bigip_ltm_pool_attachment.node-pool-attach \\
        	'{"pool": "/Common/terraform-pool", "node": "/Common/node1:80"}'
        ```

        :param str resource_name: The name of the resource.
        :param PoolAttachmentArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(PoolAttachmentArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 connection_limit: Optional[pulumi.Input[_builtins.int]] = None,
                 connection_rate_limit: Optional[pulumi.Input[_builtins.str]] = None,
                 dynamic_ratio: Optional[pulumi.Input[_builtins.int]] = None,
                 fqdn_autopopulate: Optional[pulumi.Input[_builtins.str]] = None,
                 monitor: Optional[pulumi.Input[_builtins.str]] = None,
                 node: Optional[pulumi.Input[_builtins.str]] = None,
                 pool: Optional[pulumi.Input[_builtins.str]] = None,
                 priority_group: Optional[pulumi.Input[_builtins.int]] = None,
                 ratio: Optional[pulumi.Input[_builtins.int]] = None,
                 state: Optional[pulumi.Input[_builtins.str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = PoolAttachmentArgs.__new__(PoolAttachmentArgs)

            __props__.__dict__["connection_limit"] = connection_limit
            __props__.__dict__["connection_rate_limit"] = connection_rate_limit
            __props__.__dict__["dynamic_ratio"] = dynamic_ratio
            __props__.__dict__["fqdn_autopopulate"] = fqdn_autopopulate
            __props__.__dict__["monitor"] = monitor
            if node is None and not opts.urn:
                raise TypeError("Missing required property 'node'")
            __props__.__dict__["node"] = node
            if pool is None and not opts.urn:
                raise TypeError("Missing required property 'pool'")
            __props__.__dict__["pool"] = pool
            __props__.__dict__["priority_group"] = priority_group
            __props__.__dict__["ratio"] = ratio
            __props__.__dict__["state"] = state
        super(PoolAttachment, __self__).__init__(
            'f5bigip:ltm/poolAttachment:PoolAttachment',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            connection_limit: Optional[pulumi.Input[_builtins.int]] = None,
            connection_rate_limit: Optional[pulumi.Input[_builtins.str]] = None,
            dynamic_ratio: Optional[pulumi.Input[_builtins.int]] = None,
            fqdn_autopopulate: Optional[pulumi.Input[_builtins.str]] = None,
            monitor: Optional[pulumi.Input[_builtins.str]] = None,
            node: Optional[pulumi.Input[_builtins.str]] = None,
            pool: Optional[pulumi.Input[_builtins.str]] = None,
            priority_group: Optional[pulumi.Input[_builtins.int]] = None,
            ratio: Optional[pulumi.Input[_builtins.int]] = None,
            state: Optional[pulumi.Input[_builtins.str]] = None) -> 'PoolAttachment':
        """
        Get an existing PoolAttachment resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[_builtins.int] connection_limit: Specifies a maximum established connection limit for a pool member or node.The default is 0, meaning that there is no limit to the number of connections.
        :param pulumi.Input[_builtins.str] connection_rate_limit: Specifies the maximum number of connections-per-second allowed for a pool member,The default is 0.
        :param pulumi.Input[_builtins.int] dynamic_ratio: Specifies the fixed ratio value used for a node during ratio load balancing.
        :param pulumi.Input[_builtins.str] fqdn_autopopulate: Specifies whether the system automatically creates ephemeral nodes using the IP addresses returned by the resolution of a DNS query for a node defined by an FQDN. The default is enabled
        :param pulumi.Input[_builtins.str] monitor: Specifies the health monitors that the system uses to monitor this pool member,value can be `none` (or) `default` (or) list of monitors joined with and ( ex: `/Common/test_monitor_pa_tc1 and /Common/gateway_icmp`).
        :param pulumi.Input[_builtins.str] node: Pool member address/fqdn with service port, (ex: `1.1.1.1:80/www.google.com:80`). (Note: Member will be in same partition of Pool)
        :param pulumi.Input[_builtins.str] pool: Name of the pool to which members should be attached,it should be "full path".The full path is the combination of the partition + name of the pool.(For example `/Common/my-pool`) or partition + directory + name of the pool (For example `/Common/test/my-pool`).When including directory in fullpath we have to make sure it is created in the given partition before using it.
        :param pulumi.Input[_builtins.int] priority_group: Specifies a number representing the priority group for the pool member. The default is 0, meaning that the member has no priority
        :param pulumi.Input[_builtins.int] ratio: "Specifies the ratio weight to assign to the pool member. Valid values range from 1 through 65535. The default is 1, which means that each pool member has an equal ratio proportion.".
        :param pulumi.Input[_builtins.str] state: Specifies the state the pool member should be in,value can be `enabled` (or) `disabled` (or) `forced_offline`).
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _PoolAttachmentState.__new__(_PoolAttachmentState)

        __props__.__dict__["connection_limit"] = connection_limit
        __props__.__dict__["connection_rate_limit"] = connection_rate_limit
        __props__.__dict__["dynamic_ratio"] = dynamic_ratio
        __props__.__dict__["fqdn_autopopulate"] = fqdn_autopopulate
        __props__.__dict__["monitor"] = monitor
        __props__.__dict__["node"] = node
        __props__.__dict__["pool"] = pool
        __props__.__dict__["priority_group"] = priority_group
        __props__.__dict__["ratio"] = ratio
        __props__.__dict__["state"] = state
        return PoolAttachment(resource_name, opts=opts, __props__=__props__)

    @_builtins.property
    @pulumi.getter(name="connectionLimit")
    def connection_limit(self) -> pulumi.Output[_builtins.int]:
        """
        Specifies a maximum established connection limit for a pool member or node.The default is 0, meaning that there is no limit to the number of connections.
        """
        return pulumi.get(self, "connection_limit")

    @_builtins.property
    @pulumi.getter(name="connectionRateLimit")
    def connection_rate_limit(self) -> pulumi.Output[_builtins.str]:
        """
        Specifies the maximum number of connections-per-second allowed for a pool member,The default is 0.
        """
        return pulumi.get(self, "connection_rate_limit")

    @_builtins.property
    @pulumi.getter(name="dynamicRatio")
    def dynamic_ratio(self) -> pulumi.Output[_builtins.int]:
        """
        Specifies the fixed ratio value used for a node during ratio load balancing.
        """
        return pulumi.get(self, "dynamic_ratio")

    @_builtins.property
    @pulumi.getter(name="fqdnAutopopulate")
    def fqdn_autopopulate(self) -> pulumi.Output[Optional[_builtins.str]]:
        """
        Specifies whether the system automatically creates ephemeral nodes using the IP addresses returned by the resolution of a DNS query for a node defined by an FQDN. The default is enabled
        """
        return pulumi.get(self, "fqdn_autopopulate")

    @_builtins.property
    @pulumi.getter
    def monitor(self) -> pulumi.Output[_builtins.str]:
        """
        Specifies the health monitors that the system uses to monitor this pool member,value can be `none` (or) `default` (or) list of monitors joined with and ( ex: `/Common/test_monitor_pa_tc1 and /Common/gateway_icmp`).
        """
        return pulumi.get(self, "monitor")

    @_builtins.property
    @pulumi.getter
    def node(self) -> pulumi.Output[_builtins.str]:
        """
        Pool member address/fqdn with service port, (ex: `1.1.1.1:80/www.google.com:80`). (Note: Member will be in same partition of Pool)
        """
        return pulumi.get(self, "node")

    @_builtins.property
    @pulumi.getter
    def pool(self) -> pulumi.Output[_builtins.str]:
        """
        Name of the pool to which members should be attached,it should be "full path".The full path is the combination of the partition + name of the pool.(For example `/Common/my-pool`) or partition + directory + name of the pool (For example `/Common/test/my-pool`).When including directory in fullpath we have to make sure it is created in the given partition before using it.
        """
        return pulumi.get(self, "pool")

    @_builtins.property
    @pulumi.getter(name="priorityGroup")
    def priority_group(self) -> pulumi.Output[_builtins.int]:
        """
        Specifies a number representing the priority group for the pool member. The default is 0, meaning that the member has no priority
        """
        return pulumi.get(self, "priority_group")

    @_builtins.property
    @pulumi.getter
    def ratio(self) -> pulumi.Output[_builtins.int]:
        """
        "Specifies the ratio weight to assign to the pool member. Valid values range from 1 through 65535. The default is 1, which means that each pool member has an equal ratio proportion.".
        """
        return pulumi.get(self, "ratio")

    @_builtins.property
    @pulumi.getter
    def state(self) -> pulumi.Output[Optional[_builtins.str]]:
        """
        Specifies the state the pool member should be in,value can be `enabled` (or) `disabled` (or) `forced_offline`).
        """
        return pulumi.get(self, "state")

